#!/bin/bash
#
# Note on PPPD settings:
# - Make sure the noauth option is set (instead of auth)
# - Make sure DNS servers are defined (add ms-dns 1.2.3.4 twice)

# SET MODEM SETTINGS HERE
# -----------------------

# Variable: serport
# serport specifies which local serial device to use.
# For example, using "ttyAMA0" will tell the script
# to use /dev/ttyAMA0 for communication.
#
# Default:
# serport=ttyUSB0
#
serport=ttyUSB0

# Variable: baud
# baud will tell the script to open the serial port at
# specified symbol rate. When connecting, make sure
# your client computer uses the same baud than what
# has been specified here.
#
# - Common baud rates: 9600, 19200, 38400, 57600
#
# Default:
# baud=57600
#
baud=57600

# Variable: echoser
# echoser sets the default behaviour of echoing serial
# data back to the client terminal. The default is 1.
#
echoser=1

# Script version
vmodver=1.0

# EXPORT SHELL VARS
# -----------------
export serport
export baud

# DEFINE FUNCTIONS
# ----------------
#
#INITIALIZE SERIAL SETTINGS
ttyinit () {
  stty -F /dev/$serport $baud
  stty -F /dev/$serport sane
  stty -F /dev/$serport raw
  stty -F /dev/$serport -echo -icrnl clocal
}

# SEND MESSAGE ON SCREEN AND OVER SERIAL
sendtty () {
  echo "$1";
  echo -en "$1\x0d\x0a" >/dev/$serport
}

# Open serial port for use. Allocate file descriptor
# and treat the serial port as a file.
ttyinit
exec 99<>/dev/$serport

sendtty ""
sendtty "Virtual Modem bootstrap for PPP link v$vmodver"
sendtty "Connection speed set to $baud baud"
sendtty ""
sendtty "TYPE HELP FOR COMMANDS"
sendtty "READY."

# MAIN LOOP
while [ "$continue" != "1" ]; do
  charhex=`head -c 1 /dev/$serport | xxd -p -`
  char="`echo -e "\x$charhex"`"

  #ECHO SERIAL INPUT TO TTY
  echo -n "$char"

  #ECHO SERIAL INPUT
  if [ "$echoser" = "1" ]; then echo -n $char > /dev/$serport; fi

  #CHECK IF NEWLINE IS SENT
  if [ "$charhex" = "0d" -o "$charhex" = "0a" ]; then
    line=$buffer
    # PARSE COMMAND
    cmd=`echo -en $buffer | tr a-z A-Z`
    buffer=
    char=

    #NEWLINE SENT - ECHO NEWLINE TO CONSOLE
    if [ "$echoser" = "1" ]; then sendtty; fi

    # DUMMY HAYES COMMAND PROCESSOR
    if [[ $cmd == AT    ]] ||
       [[ $cmd == ATA*  ]] ||
       [[ $cmd == ATH*  ]] ||
       [[ $cmd == ATL*  ]] ||
       [[ $cmd == ATM*  ]] ||
       [[ $cmd == ATS*  ]] ||
       [[ $cmd == ATV*  ]] ||
       [[ $cmd == ATX*  ]] ||
       [[ $cmd == AT\&* ]]; then sendtty "OK"
    fi

    # ATE  -  TERMINAL ECHO
    if [[ $cmd == ATE*  ]]; then
      if [[ $cmd == ATE1* ]]; then echoser=1; else echoser=0; sendtty "OK"; fi
    fi
    if [[ $cmd == ATZ* ]]; then echoser=1; sendtty "OK"; fi

    if [[ $cmd = HELP ]]; then
      sendtty "Command Reference for Virtual Modem Bootstrap v$vmodver"
      sendtty
      sendtty "AT......Tests modem link, prints OK if successful"
      sendtty "ATE0....Switch terminal echo off"
      sendtty "ATE1....Switch terminal echo on"
      sendtty "ATD?....Fork program ?.sh and output on terminal"
      sendtty "ATDT1...Open PPPD connection"
      sendtty "ATZ.....Reset settings"
      sendtty "HELP....Display command reference"
      sendtty "LOGIN...Fork a new linux login on serial"
      sendtty "EXIT....End this script"
      sendtty
      sendtty "To establish connection over PPP, dial 1 using tone dialing (ATDT1)"
      sendtty
      sendtty "READY."
    fi

    # ATD  -  DIAL
    if [[ $cmd == ATD* ]]; then
      sendtty "RINGING"; sendtty; sleep 1
      if [ -f "`echo $cmd |cut -b 4-`.sh" ]; then
        sendtty "CONNECT $baud"
        # Close serial port
        exec 99>&-
        # Execute dialed script
        /sbin/getty -8 -L $serport $baud vt100 -n -l "./`echo $cmd|cut -b 4-`.sh"
        # Reset serial settings
        ttyinit
        # Reopen serial port
        exec 99<>/dev/$serport
      fi
      sendtty "NO CARRIER"
    fi

    # LOGIN  -  FORK LOGIN SESSION
    if [[ $cmd == LOGIN ]]; then
      exec 99>&-
      /sbin/getty -8 -L ttyAMA0 $baud vt100
      ttyinit
      exec 99<>/dev/$serport
      sendtty; sendtty "READY."
    fi

    # EXIT  -  EXIT SCRIPT
    if [ "$cmd" = "EXIT" ]; then sendtty "OK"; continue="1"; fi
  fi
  buffer=$buffer$char
done

#Close serial port
exec 99>&-